from typing import Final
from .dassana_env import *
import json
import logging
import asyncio
import ssl
import nats

logger: Final = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO)

async def publish_message_nats(message, nats_partner_subject_name):
  try:
    data = json.dumps(message)
    
    ssl_ctx = ssl.create_default_context()
    ssl_ctx.check_hostname = False
    ssl_ctx.verify_mode = ssl.CERT_NONE

    nats_client = await nats.connect(get_nats_url(), tls=ssl_ctx)
    jet_stream = nats_client.jetstream()

    await jet_stream.publish(nats_partner_subject_name, data.encode("utf-8"))
    
    await nats_client.flush()
    await nats_client.drain()
    await nats_client.close()

  except Exception as e:
    logger.error(f"Failed To Publish Message to nats-subject {nats_partner_subject_name} Because of {e}")


import os

def get_app_id():
    if "DASSANA_APP_ID" not in os.environ:
        raise KeyError(
            "DASSANA_APP_ID environment variable is not set. Review your Lambda configuration."
        )
    return os.environ["DASSANA_APP_ID"]

def get_auth_url():
    if "DASSANA_AUTH_URL" not in os.environ:
        return "https://auth.dassana.cloud"
    return os.environ["DASSANA_AUTH_URL"]

def get_app_url():
    if "DASSANA_APP_SERVICE_URL" not in os.environ:
        return "app-manager.dassana.cloud"
    return os.environ["DASSANA_APP_SERVICE_URL"]

def get_tenant_id():
    if not is_internal_auth():
        return ""
    if "DASSANA_TENANT_ID" not in os.environ:
        raise KeyError(
            "DASSANA_TENANT_ID environment variable is not set. Review your Lambda configuration."
        )
    return os.environ["DASSANA_TENANT_ID"]

def get_if_debug():
    return int(os.environ.get("DASSANA_DEBUG", 0))

def get_ingestion_srv_url():
    if "DASSANA_INGESTION_SERVICE_URL" not in os.environ:
        return "https://ingestion-srv.dassana.cloud"
    return os.environ["DASSANA_INGESTION_SERVICE_URL"]

def get_client_id():
    if "DASSANA_CLIENT_ID" not in os.environ:
        raise KeyError(
            "DASSANA_CLIENT_ID environment variable is not set. Review your Lambda configuration."
        )
    return os.environ["DASSANA_CLIENT_ID"]

def get_client_secret():
    if "DASSANA_CLIENT_SECRET" in os.environ:
        return os.environ["DASSANA_CLIENT_SECRET"]
    elif "dassana.auth.client-secret" in os.environ:
        return os.environ["dassana.auth.client-secret"]
    else:
        raise KeyError(
                "'dassana.auth.client-secret' environment variable is not set. Review your Lambda configuration."
            )

def get_ingestion_config_id():
    if "DASSANA_INGESTION_CONFIG_ID" not in os.environ:
        raise KeyError(
            "DASSANA_INGESTION_CONFIG_ID environment variable is not set. Review your Lambda configuration."
        )
    return str(os.environ["DASSANA_INGESTION_CONFIG_ID"])

def get_dassana_token():
    if "DASSANA_TOKEN" not in os.environ:
        raise KeyError(
            "DASSANA_TOKEN environment variable is not set. Review your Lambda configuration."
        )
    return str(os.environ["DASSANA_TOKEN"])

def is_internal_auth():
    try:
        get_dassana_token()
    except KeyError:
        return True
    return False

# def get_project_id():
#     if "GCP_PROJECT_ID" not in os.environ:
#         raise KeyError(
#             "GCP_PROJECT_ID environment variable is not set. Review your configuration."
#         )
#     return str(os.getenv("GCP_PROJECT_ID"))

def get_partner():
    if "DASSANA_PARTNER" not in os.environ:
        return None
    return str(os.environ["DASSANA_PARTNER"])

def get_partner_client_id():
    if "DASSANA_PARTNER_CLIENT_ID" not in os.environ:
        return None
    return str(os.environ["DASSANA_PARTNER_CLIENT_ID"])

def get_partner_tenant_id():
    if "DASSANA_PARTNER_TENANT_ID" not in os.environ:
        return None
    return str(os.environ["DASSANA_PARTNER_TENANT_ID"])

def get_scope_to_run():
    if "scope_to_run" not in os.environ:
        return None
    return str(os.environ["scope_to_run"])

def get_nats_url():
    return os.environ.get("DASSANA_NATS_URL", "nats://nats.nats.svc:4222")


import ujson as json
from requests.models import Response


class DassanaException(Exception):
    """Exception Raised when something bad happened within dassana and could not auto recover"""
    is_internal = True
    is_auto_recoverable = False
    error_type = "internal_error"
    message = "Unexpected error occurred while collecting data"

    def __init__(self, error_details):
        super().__init__()
        self.error_details = error_details

    def __str__(self):
        error_msg = f"[{self.error_type}] {self.message}"
        if self.error_details is not None:
            error_msg += f" : {self.error_details}"
        return error_msg

    def to_json(self):
        return json.dumps(self.__dict__)


class ApiRequest:
    method = None
    url = None
    body = None

    def __init__(self, method, url, body=None):
        self.method = method
        self.url = url
        if body is not None:
            self.body = body

    @classmethod
    def from_request(cls, request):
        return cls(request.url, request.body)

    def __str__(self):
        request_str = f"URL: {self.url} "
        if self.body is not None:
            request_str += f"Body: {self.body} "
        return request_str

    def to_json(self):
        return json.dumps(self.__dict__)


class ApiResponse:
    status_code = 0
    status_message = None
    headers = None
    body = None

    def __init__(self, status_code=0, status_message=None, headers=None, body=None):
        if status_code is not None:
            self.status_code = status_code
        if status_message is not None:
            self.status_message = status_message
        if headers is not None:
            self.headers = dict(headers)
        if body is not None:
            self.body = body

    @classmethod
    def from_response(cls, response: Response):
        return cls(response.status_code, response.reason, response.headers, response.text)

    def __str__(self):
        response_str = f"Status code: {self.status_code} "
        if self.status_message is not None:
            response_str += f"Reason: {self.status_message} "
        if self.body is not None:
            response_str += f"Body: {self.body} "
        if self.headers is not None:
            response_str += f"Headers: {self.headers}"
        return response_str

    def __json__(self):
        return json.dumps(self.__dict__)


class ApiError(DassanaException):
    """Exception Raised when api request failed"""
    http_response = None

    def __init__(self, http_request: ApiRequest, http_response: ApiResponse, is_internal, error_type="internal_error",
                 error_details=None, is_auto_recoverable=False):
        super().__init__(error_details)
        self.http_request = http_request
        if http_response is not None:
            self.http_response = http_response
        self.error_type = error_type
        self.is_internal = is_internal
        self.is_auto_recoverable = is_auto_recoverable

    def __str__(self):
        error_str = f"[{self.error_type}] : API Request failed - {self.http_request} "
        if self.http_response is not None:
            error_str += f"Response - {self.http_response} "
        if self.error_details is not None:
            error_str += f"due to {self.error_details} "
        return error_str


class AuthError(ApiError):
    """Exception Raised when credentials in configuration are invalid"""
    error_type = "auth_error"
    message = "Authorization Failure"

    def __init__(self, request, response, is_internal):
        super().__init__(request, response, error_type=self.error_type, is_auto_recoverable=False, 
                         is_internal=is_internal)


class NetworkError(ApiError):
    """Exception Raised when api request failed to connection failure due to network issue or server not responding"""
    error_type = "network_error"
    message = "Connection/Network Failure"

    def __init__(self, request, error_msg, is_internal):
        super().__init__(request, ApiResponse(), error_type=self.error_type, error_details=error_msg, 
                         is_auto_recoverable=True, is_internal=is_internal)


class ServerError(ApiError):
    """Exception Raised when server responded with 5xx failure"""
    error_type = "server_error"
    message = "Server Failure"

    def __init__(self, request, response, is_internal):
        super().__init__(request, response, error_type=self.error_type, is_auto_recoverable=True,
                         is_internal=is_internal)


class RateLimitError(ApiError):
    """Exception Raised when api request is rate limited by server"""
    error_type = "rate_limit_error"
    message = "Rate limit exceeded"

    def __init__(self, request, response, is_internal):
        super().__init__(request, response, error_type=self.error_type, is_auto_recoverable=True,
                         is_internal=is_internal)


class BadResponseError(ApiError):
    """Exception Raised when api response is unexpected even if it is successful"""
    error_type = "bad_response"
    message = "Unexpected response"

    def __init__(self, request, response=None, is_internal=False, error_details=None):
        super().__init__(request, response, error_type=self.error_type, error_details=error_details, is_auto_recoverable=False,
                         is_internal=is_internal)


class ExternalError(DassanaException):
    """Exception Raised when an unexpected exception occurred in an external service"""
    is_internal = False
    is_auto_recoverable = False
    error_type = "external_error"
    message = "Something went wrong"

    def __init__(self, error_details=None):
        super().__init__(error_details)


class InternalError(DassanaException):
    """Exception Raised when something bad happened within dassana and could not auto recover"""


class StageWriteFailure(DassanaException):
    """Exception for StageWriteFailure"""
    is_internal = True
    is_auto_recoverable = False
    error_type = "stage_write_failure"

    def __init__(self, error_details=None):
        super().__init__(error_details)
